import { AwsHttpClientOptions } from './types';
import { Json } from '../../../types/json';
import { HttpClient } from '../http-client';
import { HttpMethod, RequestResponse } from '../http-client/types';
export declare class AwsHttpClient extends HttpClient<AwsHttpClientOptions> {
    /**
     * Cached credentials used to sign requests. This class will store
     * credentials while they are active and refresh the cache if needed
     * based on the `credentialsExpiration` class property.
     */
    private cachedCredentials?;
    /**
     * Expiration date of the cached credentials. This class will refresh
     * the cache if the credentials expire within 5 minutes.
     */
    private credentialsExpiration?;
    /**
     * Creates a new AWS HTTP client with SigV4 authentication.
     *
     * @param options - Configuration options for the client
     */
    constructor(options: AwsHttpClientOptions);
    /**
     * Override the sendHttpRequest method to add AWS signing
     */
    protected sendHttpRequest(url: string, method: HttpMethod, headers?: Record<string, string>, body?: Json): Promise<RequestResponse>;
    /**
     * Signs an HTTP request using AWS SigV4.
     *
     * @param request - HTTP request to sign
     * @returns Promise resolving to the signed request
     */
    private signRequest;
}
